@@ -11,34 +11,28 @@ use crate::consts::MAIN_CALLER;
 use regex::Regex;
 use crate::utils::Utils;
 use crate::processor::Processor;
+#[cfg(feature = "csv")]
 use crate::formatter::Formatter;
+#[cfg(feature = "lipsum")]
 use lipsum::lipsum;
-use lazy_static::lazy_static;
 
 // Args, greediness, processor
 type MacroType = fn(&str, bool ,&mut Processor) -> Result<String, RadError>;
 
-lazy_static!{
-   pub static ref ITER: Regex = Regex::new(r"\$:").unwrap();
-}
-
 #[derive(Clone)]
 pub struct BasicMacro {
     macros : HashMap<String, MacroType>,
 }
 
 impl BasicMacro {
+    /// Creates new basic macro hashmap
     pub fn new() -> Self {
         // Create hashmap of functions
-        let map = HashMap::from_iter(IntoIter::new([
+        let mut map = HashMap::from_iter(IntoIter::new([
             ("regex".to_owned(), BasicMacro::regex_sub as MacroType),
-            ("eval".to_owned(), BasicMacro::eval as MacroType),
             ("trim".to_owned(), BasicMacro::trim as MacroType),
             ("chomp".to_owned(), BasicMacro::chomp as MacroType).to_owned(),
             ("comp".to_owned(), BasicMacro::compress as MacroType).to_owned(),
-            ("lipsum".to_owned(), BasicMacro::placeholder as MacroType).to_owned(),
-            ("time".to_owned(), BasicMacro::time as MacroType).to_owned(),
-            ("date".to_owned(), BasicMacro::date as MacroType).to_owned(),
             ("include".to_owned(), BasicMacro::include as MacroType).to_owned(),
             ("repeat".to_owned(), BasicMacro::repeat as MacroType).to_owned(),
             ("syscmd".to_owned(), BasicMacro::syscmd as MacroType).to_owned(),
@@ -49,8 +43,6 @@ impl BasicMacro {
             ("undef".to_owned(), BasicMacro::undefine_call as MacroType).to_owned(),
             ("rename".to_owned(), BasicMacro::rename_call as MacroType).to_owned(),
             ("append".to_owned(), BasicMacro::append as MacroType).to_owned(),
-            ("from".to_owned(), BasicMacro::from_data as MacroType).to_owned(),
-            ("table".to_owned(), BasicMacro::table as MacroType).to_owned(),
             ("len".to_owned(), BasicMacro::len as MacroType).to_owned(),
             ("tr".to_owned(), BasicMacro::translate as MacroType).to_owned(),
             ("sub".to_owned(), BasicMacro::substring as MacroType).to_owned(),
@@ -64,56 +56,109 @@ impl BasicMacro {
             ("bind".to_owned(), BasicMacro::bind_to_local as MacroType).to_owned(),
             ("env".to_owned(), BasicMacro::get_env as MacroType).to_owned(),
             ("path".to_owned(), BasicMacro::merge_path as MacroType).to_owned(),
+            ("nl".to_owned(), BasicMacro::newline as MacroType).to_owned(),
             ("-".to_owned(), BasicMacro::get_pipe as MacroType).to_owned(),
         ]));
+        
+        // Optional macros
+        #[cfg(feature = "csv")]
+        {
+            map.insert("from".to_owned(), BasicMacro::from_data as MacroType);
+            map.insert("table".to_owned(), BasicMacro::table as MacroType);
+        }
+        #[cfg(feature = "chrono")]
+        {
+            map.insert("time".to_owned(), BasicMacro::time as MacroType);
+            map.insert("date".to_owned(), BasicMacro::date as MacroType);
+        }
+        #[cfg(feature = "lipsum")]
+        map.insert("lipsum".to_owned(), BasicMacro::placeholder as MacroType);
+        #[cfg(feature = "evalexpr")]
+        map.insert("eval".to_owned(), BasicMacro::eval as MacroType);
+
         // Return struct
         Self { macros : map }
     }
 
+    /// Check if a given macro exists
+    ///
+    /// # Arguments
+    ///
+    /// * `name` - Name of the macro to find
     pub fn contains(&self, name: &str) -> bool {
         self.macros.contains_key(name)
     }
 
+    /// Call a macro
+    ///
+    /// # Arguments
+    ///
+    /// * `name` - Name of the macro to call
+    /// * `args` - Argument sto supply to macro
+    /// * `greedy` - Whether macro should interpret arguments greedily
+    /// * `processor` - Processor instance to execute macro
     pub fn call(&mut self, name : &str, args: &str,greedy: bool, processor: &mut Processor) -> Result<String, RadError> {
-        let args = self.parse_inner(processor, args)?;
+        // TODO
+        // Check if this code is necessary
         if let Some(func) = self.macros.get(name) {
             // Print out macro call result
-            let result = func(&args, greedy, processor)?;
+            let result = func(args, greedy, processor)?;
             Ok(result)
         } else {
             Ok(String::new())
         }
     }
 
+    /// Undefine a macro
+    ///
+    /// # Arguments
+    ///
+    /// * `name` - Macro name to undefine
     pub fn undefine(&mut self, name: &str) {
         self.macros.remove(name);
     }
 
+    /// Rename a macro
+    ///
+    /// # Arguments
+    ///
+    /// * `name` - Source macro name to find
+    /// * `target` - Target macro name to apply
     pub fn rename(&mut self, name: &str, target: &str) {
         let func = self.macros.remove(name).unwrap();
         self.macros.insert(target.to_owned(), func);
     }
 
-    fn parse_inner(&mut self,processor : &mut Processor, target: &str) -> Result<String, RadError> {
-        processor.parse_chunk(0, &MAIN_CALLER.to_owned(), target)
-    }
-
     // ==========
     // Basic Macros
     // ==========
+    /// Print out current time
+    ///
+    /// # Usage
+    ///
     /// $time()
+    #[cfg(feature = "chrono")]
     fn time(_: &str, _: bool, _ : &mut Processor) -> Result<String, RadError> {
         Ok(format!("{}", chrono::offset::Local::now().format("%H:%M:%S")))
     }
 
+    /// Print out current date
+    ///
+    /// # Usage
+    ///
     /// $date()
+    #[cfg(feature = "chrono")]
     fn date(_: &str, _: bool, _ : &mut Processor) -> Result<String, RadError> {
         Ok(format!("{}", chrono::offset::Local::now().format("%Y-%m-%d")))
     }
 
+    /// Substitute the given source with following match expressions
+    ///
+    /// # Usage
+    ///
     /// $regex(source_text,regex_match,substitution)
     fn regex_sub(args: &str, greedy: bool, _: &mut Processor) -> Result<String, RadError> {
-        if let Some(args) = ArgParser::args_with_len(args, 3, greedy) {
+        if let Some(args) = ArgParser::new().args_with_len(args, 3, greedy) {
             let source= &args[0];
             let match_expr= &args[1];
             let substitution= &args[2];
@@ -127,10 +172,16 @@ impl BasicMacro {
         }
     }
 
-    /// $eval(expression)
+    /// Evaluate given expression
+    ///
     /// This returns true, false or evaluated number
+    ///
+    /// # Usage
+    ///
+    /// $eval(expression)
+    #[cfg(feature = "evalexpr")]
     fn eval(args: &str, greedy: bool,_: &mut Processor ) -> Result<String, RadError> {
-        if let Some(args) = ArgParser::args_with_len(args, 1, greedy) {
+        if let Some(args) = ArgParser::new().args_with_len(args, 1, greedy) {
             let formula = &args[0];
             let result = evalexpr::eval(formula)?;
             // TODO
@@ -141,20 +192,26 @@ impl BasicMacro {
         }
     }
 
-    // Trim preceding and trailing whitespaces
-    /// $trim(text)
+    /// Trim preceding and trailing whitespaces (' ', '\n', '\t', '\r')
+    ///
+    /// # Usage
+    ///
+    /// $trim(expression)
     fn trim(args: &str, greedy: bool, _: &mut Processor) -> Result<String, RadError> {
-        if let Some(args) = ArgParser::args_with_len(args, 1, greedy) {
+        if let Some(args) = ArgParser::new().args_with_len(args, 1, greedy) {
             Utils::trim(&args[0])
         } else {
             Err(RadError::InvalidArgument("Trim requires an argument".to_owned()))
         }
     }
 
-    // Remove duplicate newlines
-    /// $chomp(test)
+    /// Removes duplicate newlines whithin given input
+    ///
+    /// # Usage
+    ///
+    /// $chomp(expression)
     fn chomp(args: &str, greedy: bool, processor: &mut Processor) -> Result<String, RadError> {
-        if let Some(args) = ArgParser::args_with_len(args, 1, greedy) {
+        if let Some(args) = ArgParser::new().args_with_len(args, 1, greedy) {
             let source = &args[0];
             let reg = Regex::new(&format!(r"{0}\s*{0}", &processor.newline))?;
             let result = reg.replace_all(source, &format!("{0}{0}", &processor.newline));
@@ -165,9 +222,13 @@ impl BasicMacro {
         }
     }
 
-    /// $comp(text)
+    /// Both apply trim and chomp to given expression
+    ///
+    /// # Usage
+    ///
+    /// $comp(Expression)
     fn compress(args: &str, greedy: bool, processor: &mut Processor) -> Result<String, RadError> {
-        if let Some(args) = ArgParser::args_with_len(args, 1, greedy) {
+        if let Some(args) = ArgParser::new().args_with_len(args, 1, greedy) {
             let source = &args[0];
             // Chomp and then compress
             let result = Utils::trim(&BasicMacro::chomp(source,greedy, processor)?)?;
@@ -178,9 +239,14 @@ impl BasicMacro {
         }
     }
 
-    /// $lipsum(Number: usize)
+    /// Creates placeholder with given amount of word counts
+    ///
+    /// # Usage
+    ///
+    /// $lipsum(Number)
+    #[cfg(feature = "lipsum")]
     fn placeholder(args: &str, greedy: bool,_: &mut Processor) -> Result<String, RadError> {
-        if let Some(args) = ArgParser::args_with_len(args, 1, greedy) {
+        if let Some(args) = ArgParser::new().args_with_len(args, 1, greedy) {
             let word_count = &args[0];
             if let Ok(count) = Utils::trim(word_count)?.parse::<usize>() {
                 Ok(lipsum(count))
@@ -192,16 +258,20 @@ impl BasicMacro {
         }
     }
 
+    /// Paste given file's content
+    ///
+    /// Every macros within the file is also expanded
+    ///
+    /// # Usage
+    ///
     /// $include(path)
     fn include(args: &str, greedy: bool, processor: &mut Processor) -> Result<String, RadError> {
-        if let Some(args) = ArgParser::args_with_len(args, 1, greedy) {
+        if let Some(args) = ArgParser::new().args_with_len(args, 1, greedy) {
             let raw = Utils::trim(&args[0])?;
             let file_path = std::path::Path::new(&raw);
             if file_path.exists() { 
-                // This reservation is necessary because from_file method clears all locals
-                let reserved_local = processor.map.local.clone();
-                let result = processor.from_file(file_path, true)?;
-                processor.map.local = reserved_local;
+                processor.set_sandbox();
+                let result = processor.from_file(file_path)?;
                 Ok(result)
             } else {
                 let formatted = format!("File path : \"{}\" doesn't exist", file_path.display());
@@ -212,9 +282,13 @@ impl BasicMacro {
         }
     }
 
-    /// $repeat(count: usize,text)
+    /// Repeat given expression about given amount times
+    ///
+    /// # Usage
+    ///
+    /// $repeat(count,text)
     fn repeat(args: &str, greedy: bool,_: &mut Processor) -> Result<String, RadError> {
-        if let Some(args) = ArgParser::args_with_len(args, 2, greedy) {
+        if let Some(args) = ArgParser::new().args_with_len(args, 2, greedy) {
             let repeat_count;
             if let Ok(count) = Utils::trim(&args[0])?.parse::<usize>() {
                 repeat_count = count;
@@ -232,11 +306,17 @@ impl BasicMacro {
         }
     }
 
+    /// Call system command
+    ///
+    /// This calls via 'CMD \C' in windows platform while unix call is operated without any mediation.
+    ///
+    /// # Usage
+    ///
     /// $syscmd(system command -a arguments)
     fn syscmd(args: &str, greedy: bool,_: &mut Processor) -> Result<String, RadError> {
-        if let Some(args_content) = ArgParser::args_with_len(args, 1, greedy) {
+        if let Some(args_content) = ArgParser::new().args_with_len(args, 1, greedy) {
             let source = &args_content[0];
-            let arg_vec = ArgParser::args_to_vec(&source, ' ', GreedyState::None);
+            let arg_vec = ArgParser::new().args_to_vec(&source, ' ', GreedyState::None);
 
             let output = if cfg!(target_os = "windows") {
                 Command::new("cmd")
@@ -260,11 +340,13 @@ impl BasicMacro {
         }
     }
 
-    // Special macro
-    // Argument is expanded after vectorization
+    /// Print content according to given condition
+    /// 
+    /// # Usage 
+    ///
     /// $ifelse(evaluation, ifstate, elsestate)
     fn ifelse(args: &str, greedy: bool, _: &mut Processor) -> Result<String, RadError> {
-        if let Some(args) = ArgParser::args_with_len(args, 2, greedy) {
+        if let Some(args) = ArgParser::new().args_with_len(args, 2, greedy) {
             let boolean = &args[0];
             let if_state = &args[1];
 
@@ -289,12 +371,15 @@ impl BasicMacro {
         }
     }
 
-    // This is composite basic macro
-    // Which means this macro acts differently by the context(Processor state)
+    /// Check if macro is defined or not
+    ///
+    /// This return 'true' or 'false'
+    ///
+    /// # Usage
+    ///
     /// $ifdef(macro_name) 
-    /// This return string true or false
     fn ifdef(args: &str, greedy: bool, processor: &mut Processor) -> Result<String, RadError> {
-        if let Some(args) = ArgParser::args_with_len(args, 1, greedy) {
+        if let Some(args) = ArgParser::new().args_with_len(args, 1, greedy) {
             let name = &Utils::trim(&args[0])?;
             let map = processor.get_map();
 
@@ -309,28 +394,37 @@ impl BasicMacro {
         }
     }
 
+    /// Undefine a macro 
+    ///
+    /// 'Define' cannot be undefined because it is not actually a macro 
+    ///
+    /// # Usage
+    ///
     /// $undef(macro_name)
     fn undefine_call(args: &str, greedy: bool, processor: &mut Processor) -> Result<String, RadError> {
-        if let Some(args) = ArgParser::args_with_len(args, 1, greedy) {
+        if let Some(args) = ArgParser::new().args_with_len(args, 1, greedy) {
             let name = &Utils::trim(&args[0])?;
 
-            processor.map.undefine(name);
+            processor.get_map().undefine(name);
             Ok("".to_owned())
         } else {
             Err(RadError::InvalidArgument("Undefine requires an argument".to_owned()))
         }
     }
 
-    // $foreach()
-    // $foreach(\*a,b,c*\,$:)
+    /// Loop around given values and substitute iterators  with the value
+    ///
+    /// # Usage 
+    ///
+    /// $foreach(\*a,b,c*\,$:)
     fn foreach(args: &str, greedy: bool, processor: &mut Processor) -> Result<String, RadError> {
-        if let Some(args) = ArgParser::args_with_len(args, 2, greedy) {
+        if let Some(args) = ArgParser::new().args_with_len(args, 2, greedy) {
             let mut sums = String::new();
             let target = &args[1]; // evaluate on loop
             let loopable = &args[0];
 
             for value in loopable.split(',') {
-                let result = processor.parse_chunk(0, &MAIN_CALLER.to_owned(), &ITER.replace_all(target, value))?;
+                let result = processor.parse_chunk(0, &MAIN_CALLER.to_owned(), &target.replace("$:", value))?;
                 sums.push_str(&result);
             }
             Ok(sums)
@@ -339,9 +433,13 @@ impl BasicMacro {
         }
     }
 
-    // $forloop(1,5,$:)
+    /// For loop around given min, max value and finally substitue iterators with value
+    ///
+    /// # Usage
+    ///
+    /// $forloop(1,5,$:)
     fn forloop(args: &str, greedy: bool, processor: &mut Processor) -> Result<String, RadError> {
-        if let Some(args) = ArgParser::args_with_len(args, 3, greedy) {
+        if let Some(args) = ArgParser::new().args_with_len(args, 3, greedy) {
             let mut sums = String::new();
             let expression = &args[2]; // evaluate on loop
 
@@ -353,9 +451,9 @@ impl BasicMacro {
             if let Ok(num) = Utils::trim(&args[1])?.parse::<usize>() {
                 max = num
             } else { return Err(RadError::InvalidArgument("Forloop's min value should be non zero positive integer".to_owned())); }
-
+            
             for value in min..=max {
-                let result = processor.parse_chunk(0, &MAIN_CALLER.to_owned(), &ITER.replace_all(expression, &value.to_string()))?;
+                let result = processor.parse_chunk(0, &MAIN_CALLER.to_owned(), &expression.replace("$:", &value.to_string()))?;
                 sums.push_str(&result);
             }
 
@@ -365,9 +463,15 @@ impl BasicMacro {
         }
     }
 
-    // $from(\*1,2,3\n4,5,6*\, macro_name)
+    /// Create multiple macro executions from given csv value
+    ///
+    /// # Usage
+    ///
+    /// $from(\*1,2,3
+    /// 4,5,6*\, macro_name)
+    #[cfg(feature = "csv")]
     fn from_data(args: &str, greedy: bool, processor: &mut Processor) -> Result<String, RadError> {
-        if let Some(args) = ArgParser::args_with_len(args, 2, greedy) {
+        if let Some(args) = ArgParser::new().args_with_len(args, 2, greedy) {
             let macro_data = &args[0];
             let macro_name = &Utils::trim(&args[1])?;
 
@@ -380,9 +484,17 @@ impl BasicMacro {
         }
     }
 
-    /// $table(github,"1,2,3\n4,5,6")
+    /// Create a table with given format and csv input
+    ///
+    /// Available formats are 'github', 'wikitext' and 'html'
+    ///
+    /// # Usage
+    ///
+    /// $table(github,"1,2,3
+    /// 4,5,6")
+    #[cfg(feature = "csv")]
     fn table(args: &str, greedy: bool, p: &mut Processor) -> Result<String, RadError> {
-        if let Some(args) = ArgParser::args_with_len(args, 2, greedy) {
+        if let Some(args) = ArgParser::new().args_with_len(args, 2, greedy) {
             let table_format = &args[0]; // Either gfm, wikitex, latex, none
             let csv_content = &args[1];
             let result = Formatter::csv_to_table(table_format, csv_content, &p.newline)?;
@@ -392,31 +504,58 @@ impl BasicMacro {
         }
     }
 
+    /// Put value into a temporary stack called pipe
+    ///
+    /// Piped value can be popped with macro '-'
+    ///
+    /// # Usage
+    ///
     /// $pipe(Value)
     fn pipe(args: &str, greedy: bool, processor: &mut Processor) -> Result<String, RadError> {
-        if let Some(args) = ArgParser::args_with_len(args, 1, greedy) {
+        if let Some(args) = ArgParser::new().args_with_len(args, 1, greedy) {
             processor.pipe_value = args[0].to_owned();
         }
         Ok(String::new())
     }
 
+    /// Bind a local macro
+    ///
+    /// Bound macro gets deleted after macro execution
+    ///
+    /// # Usage
+    ///
     /// $bind(name,value)
     fn bind_to_local(args: &str, greedy: bool, processor: &mut Processor) -> Result<String, RadError> {
-        if let Some(args) = ArgParser::args_with_len(args, 2, greedy) {
+        if let Some(args) = ArgParser::new().args_with_len(args, 2, greedy) {
             let name = &args[0];
             let value = &args[1];
-            processor.map.new_local(1, name, value);
+            processor.get_map().new_local(1, name, value);
+            if processor.get_map().custom.contains_key(name) {
+                processor.log_warning(&format!("Creating a binding with a name already existing : \"{}\"", name))?;
+            }
         }
         Ok(String::new())
     }
 
+    /// Get environment variable with given name
+    ///
+    /// # Usage
+    ///
+    /// $env(SHELL)
     fn get_env(args: &str, _: bool, _: &mut Processor) -> Result<String, RadError> {
         let out = std::env::var(args)?;
         Ok(out)
     }
 
+    /// Merge two path into a single path
+    ///
+    /// This creates platform agonistic path which can be consumed by other macros.
+    ///
+    /// # Usage
+    ///
+    /// $path($env(HOME),document)
     fn merge_path(args: &str, greedy: bool, _: &mut Processor) -> Result<String, RadError> {
-        if let Some(args) = ArgParser::args_with_len(args, 2, greedy) {
+        if let Some(args) = ArgParser::new().args_with_len(args, 2, greedy) {
             let target = Utils::trim(&args[0])?;
             let added = Utils::trim(&args[1])?;
 
@@ -427,6 +566,19 @@ impl BasicMacro {
         }
     }
 
+    /// Yield newline according to platform or user option
+    ///
+    /// # Usage
+    ///
+    /// $nl()
+    fn newline(_: &str, _: bool, p: &mut Processor) -> Result<String, RadError> {
+        Ok(p.newline.to_owned())
+    }
+
+    /// Pop pipe value
+    ///
+    /// # Usage
+    ///
     /// $-()
     fn get_pipe(_: &str, _: bool, processor: &mut Processor) -> Result<String, RadError> {
         let out = processor.pipe_value.clone();
@@ -435,18 +587,29 @@ impl BasicMacro {
     }
 
     /// Return a length of the string
-    /// This is O(n) operation
+    /// 
+    /// This is O(n) operation.
     /// String.len() function returns byte length not "Character" length
     /// therefore, chars().count() is used
+    ///
+    /// # Usage
+    ///
+    /// $len(안녕하세요)
+    /// $len(Hello)
     fn len(args: &str, _: bool, _: &mut Processor) -> Result<String, RadError> {
         Ok(args.chars().count().to_string())
     }
 
+    /// Rename macro rule to other name
+    ///
+    /// # Usage
+    ///
+    /// $rename(name,target)
     fn rename_call(args: &str, greedy: bool, processor: &mut Processor) -> Result<String, RadError> {
-        if let Some(args) = ArgParser::args_with_len(args, 2, greedy) {
+        if let Some(args) = ArgParser::new().args_with_len(args, 2, greedy) {
             let target = &args[0];
             let new = &args[1];
-            processor.map.rename(target, new);
+            processor.get_map().rename(target, new);
 
             Ok(String::new())
         } else {
@@ -454,11 +617,16 @@ impl BasicMacro {
         }
     }
 
+    /// Append content to a macro
+    ///
+    /// # Usage
+    ///
+    /// $append(macro_name,Content)
     fn append(args: &str, greedy: bool, processor: &mut Processor) -> Result<String, RadError> {
-        if let Some(args) = ArgParser::args_with_len(args, 2, greedy) {
+        if let Some(args) = ArgParser::new().args_with_len(args, 2, greedy) {
             let name = &args[0];
             let target = &args[1];
-            processor.map.append(name, target);
+            processor.get_map().append(name, target);
 
             Ok(String::new())
         } else {
@@ -466,8 +634,13 @@ impl BasicMacro {
         }
     }
 
+    /// Translate given char aray into corresponding char array
+    ///
+    /// # Usage
+    ///
+    /// $tr(Source,abc,ABC)
     fn translate(args: &str, greedy: bool, _: &mut Processor) -> Result<String, RadError> {
-        if let Some(args) = ArgParser::args_with_len(args, 3, greedy) {
+        if let Some(args) = ArgParser::new().args_with_len(args, 3, greedy) {
             let mut source = args[0].clone();
             let target = &args[1].chars().collect::<Vec<char>>();
             let destination = &args[2].chars().collect::<Vec<char>>();
@@ -486,9 +659,13 @@ impl BasicMacro {
         }
     }
 
-    // $sub(0,5,GivenString)
+    /// Get a substring(indexed) from given source
+    ///
+    /// # Usage
+    ///
+    /// $sub(0,5,GivenString)
     fn substring(args: &str, greedy: bool, _: &mut Processor) -> Result<String, RadError> {
-        if let Some(args) = ArgParser::args_with_len(args, 2, greedy) {
+        if let Some(args) = ArgParser::new().args_with_len(args, 2, greedy) {
             let source = &args[2];
 
             let mut min: Option<usize> = None;
@@ -519,8 +696,17 @@ impl BasicMacro {
             Err(RadError::InvalidArgument("Sub requires some arguments".to_owned()))
         }
     }
+
+    /// Pause every macro expansion
+    ///
+    /// Only other pause call is evaluated
+    ///
+    /// # Usage
+    /// 
+    /// $pause(true)
+    /// $pause(false)
     fn pause(args: &str, greedy: bool, processor : &mut Processor) -> Result<String, RadError> {
-        if let Some(args) = ArgParser::args_with_len(args, 1, greedy) {
+        if let Some(args) = ArgParser::new().args_with_len(args, 1, greedy) {
             let arg = &args[0];
             if let Ok(value) =Utils::is_arg_true(arg) {
                 if value {
@@ -539,8 +725,13 @@ impl BasicMacro {
         }
     }
 
+    /// Save content to temporary file
+    ///
+    /// # Usage
+    ///
+    /// $tempout(true,Content)
     fn temp_out(args: &str, greedy: bool, p: &mut Processor) -> Result<String, RadError> {
-        if let Some(args) = ArgParser::args_with_len(args, 1, greedy) {
+        if let Some(args) = ArgParser::new().args_with_len(args, 1, greedy) {
             let content = &args[0];
             p.get_temp_file().write_all(content.as_bytes())?;
             Ok(String::new())
@@ -549,8 +740,13 @@ impl BasicMacro {
         }
     }
 
+    /// Save content to a file
+    ///
+    /// # Usage
+    ///
+    /// $fileout(true,file_name,Content)
     fn file_out(args: &str, greedy: bool, _: &mut Processor) -> Result<String, RadError> {
-        if let Some(args) = ArgParser::args_with_len(args, 3, greedy) {
+        if let Some(args) = ArgParser::new().args_with_len(args, 3, greedy) {
             let truncate = &args[0];
             let file_name = &args[1];
             let content = &args[2];
@@ -580,13 +776,27 @@ impl BasicMacro {
         }
     }
 
+    /// Include but for temporary file
+    ///
+    /// # Usage
+    ///
+    /// $tempin()
     fn temp_include(_: &str, _: bool, processor: &mut Processor) -> Result<String, RadError> {
         let file = processor.get_temp_path().to_owned();
-        Ok(processor.from_file(&file, true)?)
+        processor.set_sandbox();
+        Ok(processor.from_file(&file)?)
     }
 
+    /// Redirect all text into temporary file
+    ///
+    /// Every text including non macro calls are all sent to current temporary files.
+    ///
+    /// # Usage
+    ///
+    /// $redir(true) 
+    /// $redir(false) 
     fn temp_redirect(args: &str, _: bool, processor: &mut Processor) -> Result<String, RadError> {
-        if let Some(args) = ArgParser::args_with_len(args, 1, false) {
+        if let Some(args) = ArgParser::new().args_with_len(args, 1, false) {
             let toggle = if let Ok(toggle) =Utils::is_arg_true(&args[0]) { 
                 toggle
             } else {
@@ -599,8 +809,13 @@ impl BasicMacro {
         }
     }
 
+    /// Set temporary file
+    ///
+    /// # Usage
+    ///
+    /// $tempto(file_name)
     fn set_temp_target(args: &str, greedy: bool, processor: &mut Processor) -> Result<String, RadError> {
-        if let Some(args) = ArgParser::args_with_len(args, 1, greedy) {
+        if let Some(args) = ArgParser::new().args_with_len(args, 1, greedy) {
             processor.set_temp_file(&PathBuf::from(std::env::temp_dir()).join(&args[0]));
             Ok(String::new())
         } else {
