@@ -93,8 +93,8 @@ impl Cli {
 
     /// Creates argument template
     fn args_builder() -> clap::ArgMatches {
-        clap_app!(R4d =>
-            (version: "0.5.1")
+        clap_app!(rad =>
+            (version: "0.5.2")
             (author: "Simon Creek <simoncreek@tutanota.com>")
             (about: "R4d is a modern macro processor made with rust")
             (@arg FILE: ... "Files to execute processing")
